@@ -37,7 +37,9 @@
 #include "extra.h"
 #include "imports.h"
 
-#define DEBUG_PREPROC
+#ifdef WITH_XSLT_DEBUG
+#define WITH_XSLT_DEBUG_PREPROC
+#endif
 
 
 /************************************************************************
@@ -160,6 +162,8 @@ xsltFreeStylePreComp(xsltStylePreCompPtr comp) {
 	xmlFree(comp->numdata.format);
     if (comp->comp != NULL)
 	xmlXPathFreeCompExpr(comp->comp);
+    if (comp->nsList != NULL)
+	xmlFree(comp->nsList);
 
     memset(comp, -1, sizeof(xsltStylePreComp));
 
@@ -195,7 +199,7 @@ xsltDocumentComp(xsltTransformContextPtr ctxt, xmlNodePtr inst) {
     comp->ver11 = 0;
 
     if (xmlStrEqual(inst->name, (const xmlChar *) "output")) {
-#ifdef DEBUG_EXTRA
+#ifdef WITH_XSLT_DEBUG_EXTRA
 	xsltGenericDebug(xsltGenericDebugContext,
 	    "Found saxon:output extension\n");
 #endif
@@ -203,7 +207,7 @@ xsltDocumentComp(xsltTransformContextPtr ctxt, xmlNodePtr inst) {
 			 (const xmlChar *)"file",
 			 XSLT_SAXON_NAMESPACE, &comp->has_filename);
     } else if (xmlStrEqual(inst->name, (const xmlChar *) "write")) {
-#ifdef DEBUG_EXTRA
+#ifdef WITH_XSLT_DEBUG_EXTRA
 	xsltGenericDebug(xsltGenericDebugContext,
 	    "Found xalan:write extension\n");
 #endif
@@ -215,7 +219,7 @@ xsltDocumentComp(xsltTransformContextPtr ctxt, xmlNodePtr inst) {
 			 (const xmlChar *)"href",
 			 XSLT_XT_NAMESPACE, &comp->has_filename);
 	if (filename == NULL) {
-#ifdef DEBUG_EXTRA
+#ifdef WITH_XSLT_DEBUG_EXTRA
 	    xsltGenericDebug(xsltGenericDebugContext,
 		"Found xslt11:document construct\n");
 #endif
@@ -224,7 +228,7 @@ xsltDocumentComp(xsltTransformContextPtr ctxt, xmlNodePtr inst) {
 			     XSLT_NAMESPACE, &comp->has_filename);
 	    comp->ver11 = 1;
 	} else {
-#ifdef DEBUG_EXTRA
+#ifdef WITH_XSLT_DEBUG_EXTRA
 	    xsltGenericDebug(xsltGenericDebugContext,
 		"Found xt:document extension\n");
 #endif
@@ -921,6 +925,8 @@ xsltStylePreCompute(xsltTransformContextPtr ctxt, xmlNodePtr inst) {
     if (inst->_private != NULL) 
         return;
     if (IS_XSLT_ELEM(inst)) {
+	xsltStylePreCompPtr cur;
+
 	if (IS_XSLT_NAME(inst, "apply-templates")) {
 	    xsltApplyTemplatesComp(ctxt, inst);
 	} else if (IS_XSLT_NAME(inst, "value-of")) {
@@ -968,6 +974,21 @@ xsltStylePreCompute(xsltTransformContextPtr ctxt, xmlNodePtr inst) {
 	    xsltGenericError(xsltGenericDebugContext,
 		 "xsltStylePreCompute: unknown xslt:%s\n", inst->name);
 	}
+	/*
+	 * Add the namespace lookup here, this code can be shared by
+	 * all precomputations.
+	 */
+	cur = inst->_private;
+	if (cur != NULL) {
+	    int i = 0;
+
+	    cur->nsList = xmlGetNsList(inst->doc, inst);
+            if (cur->nsList != NULL) {
+		while (cur->nsList[i] != NULL)
+		    i++;
+	    }
+	    cur->nsNr = i;
+	}
     } else {
 	if (IS_XSLT_NAME(inst, "document")) {
 	    xsltDocumentComp(ctxt, inst);
