@@ -28,7 +28,9 @@
 #include "namespaces.h"
 #include "attributes.h"
 
-#define DEBUG_TEMPLATES
+#ifdef WITH_XSLT_DEBUG
+#define WITH_XSLT_DEBUG_TEMPLATES
+#endif
 
 /************************************************************************
  *									*
@@ -54,17 +56,20 @@ xsltEvalXPathPredicate(xsltTransformContextPtr ctxt,
 
     position = ctxt->xpathCtxt->proximityPosition;
     ctxt->xpathCtxt->node = ctxt->node;
+    /* TODO: do we need to propagate the namespaces here ? */
+    ctxt->xpathCtxt->namespaces = NULL;
+    ctxt->xpathCtxt->nsNr = 0;
     res = xmlXPathCompiledEval(comp, ctxt->xpathCtxt);
     ctxt->xpathCtxt->proximityPosition = position;
     if (res != NULL) {
 	ret = xmlXPathEvalPredicate(ctxt->xpathCtxt, res);
 	xmlXPathFreeObject(res);
-#ifdef DEBUG_TEMPLATES
+#ifdef WITH_XSLT_DEBUG_TEMPLATES
 	xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltEvalXPathPredicate: returns %d\n", ret);
 #endif
     } else {
-#ifdef DEBUG_TEMPLATES
+#ifdef WITH_XSLT_DEBUG_TEMPLATES
 	xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltEvalXPathPredicate: failed\n");
 #endif
@@ -89,6 +94,9 @@ xsltEvalXPathString(xsltTransformContextPtr ctxt, xmlXPathCompExprPtr comp) {
     xmlXPathObjectPtr res;
 
     ctxt->xpathCtxt->node = ctxt->node;
+    /* TODO: do we need to propagate the namespaces here ? */
+    ctxt->xpathCtxt->namespaces = NULL;
+    ctxt->xpathCtxt->nsNr = 0;
     res = xmlXPathCompiledEval(comp, ctxt->xpathCtxt);
     if (res != NULL) {
 	if (res->type != XPATH_STRING)
@@ -102,7 +110,7 @@ xsltEvalXPathString(xsltTransformContextPtr ctxt, xmlXPathCompExprPtr comp) {
 	}
 	xmlXPathFreeObject(res);
     }
-#ifdef DEBUG_TEMPLATES
+#ifdef WITH_XSLT_DEBUG_TEMPLATES
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltEvalXPathString: returns %s\n", ret);
 #endif
@@ -243,7 +251,7 @@ xsltEvalAttrValueTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
      */
 
     ret = xsltAttrTemplateValueProcess(ctxt, expr);
-#ifdef DEBUG_TEMPLATES
+#ifdef WITH_XSLT_DEBUG_TEMPLATES
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltEvalXPathString: %s returns %s\n", expr, ret);
 #endif
